/* src/interfaces/ecpg/include/datetime.h */

#ifndef _ECPG_DATETIME_H
#define _ECPG_DATETIME_H

#include <ecpg_informix.h>

/* source created by ecpg which defines these symbols */
#ifndef _ECPGLIB_H
typedef timestamp dtime_t;
typedef interval intrvl_t;
#endif   /* ndef _ECPGLIB_H */

#endif   /* ndef _ECPG_DATETIME_H */
